@@ -5,7 +5,8 @@ mosaic <- function(x, ...)
   UseMethod("mosaic")
 
 mosaic.formula <-
-function(formula, data = NULL, ..., main = NULL, sub = NULL, subset = NULL)
+function(formula, data = NULL, highlighting = NULL,
+         ..., main = NULL, sub = NULL, subset = NULL, na.action = NULL)
 {
   if (is.logical(main) && main)
     main <- deparse(substitute(data))
@@ -18,7 +19,23 @@ function(formula, data = NULL, ..., main = NULL, sub = NULL, subset = NULL)
   fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
   vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
   varnames <- vars[[1]]
+
   condnames <- if (length(vars) > 1) vars[[2]] else NULL
+  
+  dep <- gsub(" ", "", fstr[[1]][1])
+  if (is.null(highlighting) && (!dep %in% c("","Freq"))) {
+     if (all(varnames == ".")) {
+       varnames <- if (is.data.frame(data))
+         colnames(data)
+       else
+         names(dimnames(as.table(data)))
+       varnames <- varnames[-which(varnames %in% dep)]
+     }
+                     
+    varnames <- c(varnames, dep)
+    highlighting <- length(varnames) + length(condnames)
+  }
+    
 
   if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
     condind <- NULL
@@ -36,24 +53,35 @@ function(formula, data = NULL, ..., main = NULL, sub = NULL, subset = NULL)
       }
       dat <- margin.table(dat, ind)
     }
-    mosaic.default(dat, main = main, sub = sub,
+    mosaic.default(dat, main = main, sub = sub, highlighting = highlighting,
                    condvars = if (is.null(condind)) NULL else match(condnames, names(dimnames(dat))), ...)
   } else {
-      m <- m[c(1, match(c("formula", "data", "subset"), names(m), 0))]
+      m <- m[c(1, match(c("formula", "data", "subset", "na.action"), names(m), 0))]
       m[[1]] <- as.name("xtabs")
       m$formula <-
           formula(paste(if("Freq" %in% colnames(data)) "Freq",
                         "~",
                         paste(c(condnames, varnames), collapse = "+")))
       tab <- eval(m, parent.frame())
-      mosaic.default(tab, main = main, sub = sub, ...)  
+      mosaic.default(tab, main = main, sub = sub, highlighting = highlighting, ...)  
   }
 }
 
 mosaic.default <- function(x, condvars = NULL,
                            split_vertical = NULL, direction = NULL,
                            spacing = NULL, spacing_args = list(),
-                           zero_size = 0.5, main = NULL, sub = NULL, ...) {
+                           gp = NULL, expected = NULL, shade = NULL,
+                           highlighting = NULL, 
+                           highlighting_fill = grey.colors,
+                           highlighting_direction = NULL,
+                           zero_size = 0.5,
+                           zero_split = FALSE,
+                           zero_shade = NULL,
+                           zero_gp = gpar(col = 0),
+                           main = NULL, sub = NULL, ...) {
+  zero_shade <- !is.null(shade) && shade || !is.null(expected) || !is.null(gp)
+  if (!is.null(shade) && !shade) zero_shade = FALSE
+  
   if (is.logical(main) && main)
     main <- deparse(substitute(x))
   else if (is.logical(sub) && sub)
@@ -77,11 +105,43 @@ mosaic.default <- function(x, condvars = NULL,
   if (length(split_vertical) < dl)
     split_vertical <- rep(split_vertical, length.out = dl)
 
-  ## condvars
+  ## highlighting
+  if (!is.null(highlighting)) {
+    if (is.character(highlighting))
+      highlighting <- match(highlighting, names(dimnames(x)))
+    if (length(highlighting) > 0) {
+      if (is.character(condvars))
+        condvars <- match(condvars, names(dimnames(x)))
+      x <- if (length(condvars) > 0)
+        aperm(x, c(condvars, seq(dl)[-c(condvars,highlighting)], highlighting))
+      else
+        aperm(x, c(seq(dl)[-highlighting], highlighting))
+      if (is.null(spacing))
+        spacing <- spacing_highlighting
+      if (is.function(highlighting_fill))
+        highlighting_fill <- rev(highlighting_fill(dim(x)[dl]))
+      if (is.null(gp))
+        gp <- gpar(fill = highlighting_fill)
+      if (!is.null(highlighting_direction)) {
+        split_vertical[dl] <- highlighting_direction %in% c("left", "right")
+        if (highlighting_direction %in% c("left", "top")) {
+          ## ugly:
+          tmp <- as.data.frame.table(x)
+          tmp[,dl] <- factor(tmp[,dl], rev(levels(tmp[,dl])))
+          x <- xtabs(Freq ~ ., data = tmp)
+          gp <- gpar(fill = rev(highlighting_fill))
+        }
+      }
+    }
+  }
+  
+  ## Conditioning only
   if (!is.null(condvars)) {
     if (is.character(condvars))
       condvars <- match(condvars, names(dimnames(x)))
-    x <- aperm(x, c(condvars, seq(dl)[-condvars]))
+    if (length(condvars) > 0)
+      x <- aperm(x, c(condvars, seq(dl)[-condvars]))
+    
     if (is.null(spacing))
       spacing <- spacing_conditional
   }
@@ -89,27 +149,33 @@ mosaic.default <- function(x, condvars = NULL,
   ## spacing argument
   if (is.null(spacing))
     spacing <- if (dl < 3) spacing_equal else spacing_increase
-
+  
   strucplot(x,
             condvars = if (is.null(condvars)) NULL else length(condvars),
-            core = struc_mosaic(zero_size = zero_size),
+            core = struc_mosaic(zero_size = zero_size, zero_split = zero_split,
+              zero_shade = zero_shade, zero_gp = zero_gp),
             split_vertical = split_vertical,
             spacing = spacing,
             spacing_args = spacing_args,
+            gp = gp,
+            expected = expected,
+            shade = shade,
             main = main,
             sub = sub,
             ...)
 }
 
-struc_mosaic <- function(zero_size = 0.5)
-  function(residuals, observed, expected = NULL, spacing, gp, split_vertical) {
+struc_mosaic <- function(zero_size = 0.5, zero_split = FALSE,
+                         zero_shade = TRUE, zero_gp = gpar(col = 0))
+  function(residuals, observed, expected = NULL, spacing, gp, split_vertical, prefix = "") {
     dn <- dimnames(observed)
     dnn <- names(dn)
     dx <- dim(observed)
     dl <- length(dx)
 
     ## split workhorse
-    split <- function(x, i, name, row, col) {
+    zerostack <- character(0)
+    split <- function(x, i, name, row, col, zero) {
       cotab <- co_table(x, 1)
       margin <- sapply(cotab, sum)
       v <- split_vertical[i]
@@ -130,10 +196,19 @@ struc_mosaic <- function(zero_size = 0.5)
       row <- col <- rep.int(1, d)
       if (v) col <- 2 * 1:d - 1 else row <- 2 * 1:d - 1
       f <- if (i < dl) 
-        function(m) split(cotab[[m]], i + 1, name[m], row[m], col[m])
+        function(m) {
+          co <- cotab[[m]]
+          z <- mean(co) <= .Machine$double.eps
+          if (z && !zero && !zero_split) zerostack <<- c(zerostack, name[m])
+          split(co, i + 1, name[m], row[m], col[m], z && !zero_split)
+        }
       else
-        function(m) viewport(layout.pos.col = col[m], layout.pos.row = row[m],
-                             name = remove_trailing_comma(name[m]))
+        function(m) {
+          if (cotab[[m]] <= .Machine$double.eps && !zero)
+            zerostack <<- c(zerostack, name[m])
+          viewport(layout.pos.col = col[m], layout.pos.row = row[m],
+                   name = remove_trailing_comma(name[m]))
+        }
       vpleaves <- structure(lapply(1:d, f), class = c("vpList", "viewport"))
 
       vpTree(vproot, vpleaves)
@@ -141,7 +216,8 @@ struc_mosaic <- function(zero_size = 0.5)
 
     ## start spltting on top, creates viewport-tree
     pushViewport(split(observed + .Machine$double.eps,
-                       i = 1, name = "cell:", row = 1, col = 1))
+                       i = 1, name = paste(prefix, "cell:", sep = ""),
+                       row = 1, col = 1, zero = FALSE))
 
     ## draw rectangles
     mnames <-  apply(expand.grid(dn), 1,
@@ -149,23 +225,35 @@ struc_mosaic <- function(zero_size = 0.5)
                      )
     zeros <- observed <= .Machine$double.eps
 
-    for (i in seq(along = mnames)) {
-      seekViewport(paste("cell:", mnames[i], sep = ""))
-      gpobj <- structure(lapply(gp, function(x) x[i]), class = "gpar")
-      if (!zeros[i]) {
-        grid.rect(gp = gpobj, name = paste("rect:", mnames[i], sep = ""))
-      } else { 
-        grid.lines(x = 0.5, gp = gpobj)
-        grid.lines(y = 0.5, gp = gpobj)
-        if (zero_size > 0) {
-          grid.points(0.5, 0.5, pch = 19, size = unit(zero_size, "char"),
-                      gp = gpar(col = gp$fill[i]),
-                      name = paste("disc:", mnames[i], sep = ""))
-          grid.points(0.5, 0.5, pch = 1, size = unit(zero_size, "char"),
-                      name = paste("circle:", mnames[i], sep = ""))
-        }
+    ## draw zero cell lines
+    for (i in remove_trailing_comma(zerostack)) {
+      seekViewport(i)
+      grid.lines(x = 0.5)
+      grid.lines(y = 0.5)
+      if (!zero_shade && zero_size > 0) {
+        grid.points(0.5, 0.5, pch = 19, size = unit(zero_size, "char"),
+                    gp = zero_gp,
+                    name = paste(prefix, "disc:", mnames[i], sep = ""))
+        grid.points(0.5, 0.5, pch = 1, size = unit(zero_size, "char"),
+                    name = paste(prefix, "circle:", mnames[i], sep = ""))
       }
     }
 
+    # draw boxes
+    for (i in seq(along = mnames)) {
+      seekViewport(paste(prefix, "cell:", mnames[i], sep = ""))
+      gpobj <- structure(lapply(gp, function(x) x[i]), class = "gpar")
+      if (!zeros[i]) {
+        grid.rect(gp = gpobj, name = paste(prefix, "rect:", mnames[i], sep = ""))
+      } else { 
+        if (zero_shade && zero_size > 0) {
+          grid.points(0.5, 0.5, pch = 19, size = unit(zero_size, "char"),
+                      gp = gpar(col = gp$fill[i]),
+                      name = paste(prefix, "disc:", mnames[i], sep = ""))
+          grid.points(0.5, 0.5, pch = 1, size = unit(zero_size, "char"),
+                      name = paste(prefix, "circle:", mnames[i], sep = ""))
+        }
+      }
+    }
   }
 class(struc_mosaic) <- "grapcon_generator"
